/***************************************************
 Monitor Unit
 Sensor Aray
 By Dave Gundlach
 5/2016

 Monitor Unit monitors Sensor Aray and displays sensor data.  Unit
 has two buttons for sending a report signal and reset signal. 
 Report will initiate sensors to transmit data.  Reset will
 instruct sensors to reset counter. Buzzer signals when sensors
 are active.  Slide switch turns off battery power.

 Components:
 1 - Adafruit Feather 32u4 Radiow with RFM69HCW Module
 1 - Adafruit 1.44" Color TFT with Micro SD Socket
 2 - Momentary Contact Switch
 1 - Vibrating Mini Motor Disk
 1 - SPST Slide Switch
 1 - 3.7V LIION Battery

 Connections:
  Feather to TFT: | Feather to Button: | Feather to Buzzer:
    3V to Vin     |  GND to B1         |  3 to Red Wire
    GND to GND    |  12 to B1          |  GND to Black Wire
    SCK to SCK    |  GND to B2         | Feather to SPST Slide Switch:
    MOSI to SI    |  ~5 to B2          |  En to Switch
    ~10 to TCS    |                    |  GND to Switch
    ~11 to RST    |
    ~6 to D/C     |
    A0 to Lite    |
   
 Libraries:
  RF69 Radio: https://www.github.com/lowpowerlab/rfm69
  1.44" TFT: https://github.com/adafruit/Adafruit-ST7735-Library
  Graphics: https://github.com/adafruit/Adafruit-GFX-Library
   
 V0001 - Mod Cmd_005 to Feather w/RF69 radio and 1.44" TFT.
 v0002 - Added Reset and Report buttons
 V0003 - Added Acive Label and highlighting 
 V0004 - Changed display
 V0005 - Added Buzzer, Sreen backlight off, and Battery Power switch.
 V0006 - Changed some pin numbers (mistake soldering for enclosure).
 V0007 - Fixed soldering error and changed pin numbers back. Added Battery Monitor.
 Added TFT backlight off after timeout.
 V0008 - Code Clean Up
 
 ****************************************************/
/* Libraries */
#include <Adafruit_GFX.h>          // Core graphics library
#include <Adafruit_ST7735.h>       // TFT library
#include <SPI.h>                   // SPI for TFT & Radio
// #include <SD.h>                    // TFT SD Card Reader
#include <RFM69.h>                 //RFM69 Radio

/* Initialise TFT  */
#define TFT_CS     10
#define TFT_RST    11                        
#define TFT_DC     6 

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);

/* Initialise RF69 Radio */
#define NETWORKID     101          // Use the same value for all sensors on network
#define NODEID        0            // The unique identifier of this node
#define RECEIVER      255          // The recipient of packets (broadcast)
#define FREQUENCY     RF69_433MHZ  // 433MHZ Radio
#define ENCRYPTKEY    "whereIsTHEfoxHat"  //Encryption Key
#define IS_RFM69HCW   true 
#define RFM69_CS      8
#define RFM69_IRQ     7
#define RFM69_IRQN    4            // Pin 7 is IRQ 4!
#define RFM69_RST     4
#define VBATPIN       A9           // Battery Voltage Pin

RFM69 radio = RFM69(RFM69_CS, RFM69_IRQ, IS_RFM69HCW, RFM69_IRQN);

/* Other Pre-Processor Deifinitions  */
#define BUFFPIXEL     20
#define RESET_BTN     12
#define REPORT_BTN    5   
#define BUZZER        3   
#define SCREEN        18            // A0 Pin

/* Global Variables  */
int sensorNumber = NODEID;         // Sensor Number
int screen = 1;                    // Screen value
int numberof = 4;                  // Number of sensor in array
int comAct = 0;                    // Communication Activity
int sv[] = {0,222,222,222,222,222,222,222,222,222,222,222,222,
    222,222};                      // Sensor Values
int svLast[15];                    // Last Sensor Values
int sLast=0;                       // Last Sensor Hit
unsigned long comTimer;            // Communication Timer (used for sLast flash)
unsigned long caTimer;             // Communication Activity Timer
int color[]={0xF800,0xF800,
    0xFFFF,0xFFFF,0x07E0};         // Colors (red, red, white, white, green)
//**********************************
//  Setup
void setup() {
  Serial.begin(115200);               // Start Serial (Debug)
  Serial.println("Setup");            // Debug

  //pinMode(10, OUTPUT);                // TFT CS must be OUTPUT (done in library)
  pinMode(RESET_BTN, INPUT_PULLUP);   // Reset Button
  pinMode(REPORT_BTN,INPUT_PULLUP);   // Report Button
  pinMode(BUZZER, OUTPUT);            // Buzzer
  pinMode(SCREEN, OUTPUT);            // Screen Backlight
  
  // Setup TFT Display //
  tft.initR(INITR_144GREENTAB);       // for 1.44" TFT
   // Hard Reset the RFM module
  pinMode(RFM69_RST, OUTPUT);
  digitalWrite(RFM69_RST, HIGH);
  delay(100);
  digitalWrite(RFM69_RST, LOW);
  delay(100);
   // Initialize radio
  radio.initialize(FREQUENCY,NODEID,NETWORKID);
  if (IS_RFM69HCW) {
    radio.setHighPower();           // Only for RFM69HCW & HW!
  }
  radio.setPowerLevel(31);          // power output ranges from 0 (5dBm) to 31 (20dBm)
  radio.encrypt(ENCRYPTKEY);
  //radio.promiscuous(true);        // not needed, using Broadcast (Reciever 255)
  SPI.usingInterrupt(RFM69_IRQN);   // Required to keep radio interupt from taking SPI before it's done
  
  BscreenDisplay();                   // initialise TFT screen

  Serial.println("Setup Complete"); // debug
}
//**********************************
// Loop  //
void loop() {
  if (comTimer>millis()){           // update screen until comTimer (sLast Flash)
    sv[0]=0;                        // clear red text
    screenUpdate();                 // update screen
  }
  else{
    digitalWrite(SCREEN, LOW);      // Turn Backlight Off
  }
  if(checkRadio()){                 // check radio
    screenUpdate();                 // update screen if recived new data
  }
  if(!digitalRead(REPORT_BTN)){     // Check report button
    BscreenDisplay();
//    delay(1000);
    Report();
  }
  if(!digitalRead(RESET_BTN)){      // Check reset button
    BscreenDisplay();
//    delay(1000);
    Reset();
  }
}

//******************* Base Screen Display Function ***********************************
// This function creates the background screen dislplay
void BscreenDisplay(){
  digitalWrite(SCREEN, HIGH);        // Backlight On
  tft.fillScreen(ST7735_BLACK);
  delay(500);
  tft.setRotation(2);
  drawBat();
  if (caTimer>millis())comAct=1;
  tft.setCursor(0, 0);
  tft.setTextColor(ST7735_BLUE);
  tft.setTextSize(1);
  tft.print("    Sensor Status ");
  tft.setCursor(25,12);
  tft.setTextColor(ST7735_BLACK,ST7735_RED);  tft.setTextSize(1);
  tft.print(" Not Active ");
//  tft.setCursor(1, 24);
  tft.setTextColor(ST7735_BLUE,ST7735_BLACK);  tft.setTextSize(2);
  
  for (int i=1; i<=numberof; i++){
    tft.setCursor(4, (i*20)+10);
    tft.print(i);
    tft.drawRect(0, i*20+8,18, 20, ST7735_WHITE);
    tft.setCursor(22, (i*20)+10);
    tft.println("?  ");
    tft.drawRect(20, i*20+8,40, 20, ST7735_WHITE);
  }
  delay(1000);
}
//******************* Screen Update Function ***********************************
// This function creates the background screen dislplay

void screenUpdate(){
  digitalWrite(SCREEN, HIGH);                                        // Backlight On
  drawBat();
  if (caTimer>millis())comAct=1;                                      
  else comAct=0;                                                      
  if(comAct){                                                         // Com Active
    tft.setCursor(25,12);
    tft.setTextColor(ST7735_BLACK,ST7735_GREEN); tft.setTextSize(1);  // Green Highlight
    tft.print("   ACTIVE   ");
    digitalWrite(BUZZER, HIGH);
  }
  else{                                                                // Com Not Active
    tft.setCursor(25,12);
    tft.setTextColor(ST7735_BLACK,ST7735_RED);  tft.setTextSize(1);    // Red Highlight
    tft.print(" Not Active ");
    digitalWrite(BUZZER, LOW);
  }
  tft.setTextColor(ST7735_WHITE,ST7735_BLACK);  tft.setTextSize(2);    // White on Black
  tft.setCursor(1, 24);
  for (int i=1; i<=numberof; i++){                                     // For each sensor
    if(sv[i] != 222){                                                  // unknown no change
      if(sv[i] != svLast[i]){                                          // if changed
        sLast=i;                                                       // Last Sensor changed
        if(i==sv[0]){                                                  // active sensor
          tft.setTextColor(ST7735_RED,ST7735_GREEN);                   // Red on Green
        }
        else{                                                          // not active sensor
          tft.setTextColor(ST7735_WHITE,ST7735_GREEN);                 // White on Green         
        }
      }
      else{                                                             // if no change
        if(i==sv[0]){                                                   // active sensor
          tft.setTextColor(ST7735_RED,ST7735_BLACK);                    // Red on Black      
        }
        else{                                                           // not active sensor
          tft.setTextColor(ST7735_WHITE,ST7735_BLACK);                  // white on Black
        }
      }  
      
    }
    else{
      tft.setTextColor(ST7735_BLUE,ST7735_BLACK);  tft.setTextSize(2);    // White on Black
    }
    tft.setCursor(4, (i*20)+10);                                        // set cursor position
    tft.print(i);                                                       // print sensor number
    tft.setCursor(22, (i*20)+10);                                       // set cursor position
    if(sv[i] !=222){                                                    // sensor value known
      tft.print(sv[i]);                                                 // print sensor value
      if (sv[i]<10)tft.print("  ");                                     // fill box
      if (sv[i]>9)tft.print(" ");
    }
    
  }
  if (sLast){
    tft.setCursor(4, (sLast*20)+10);                                    // set position to last changed sensor
    tft.setTextColor(ST7735_WHITE,ST7735_GREEN);                        // highlight green
    tft.print(sLast);                                                   // print
  }
  for(int i=1; i<=numberof; i++){
    svLast[i]=sv[i];                                                    // update last sensor values
  }
}
//******************* Draw Battery Function ***********************************
// This function draws battery symbol
int drawBat(){
  int bValue = batLevel();                  // read battery level
  tft.drawRect(0,0,16,8,color[bValue]);     // draw battery body
  tft.drawRect(16,2,2,4,color[bValue]);     // draw battery terminal
  if (bValue>2){
    tft.fillRect(0,0,16,8,color[bValue]);   // fill battery body
    tft.fillRect(16,2,2,4,color[bValue]);   // fill battery terminal
  }
  if (bValue==2){
    tft.fillRect(0,0,8,8,color[bValue]);    // fill half battery body
  }
  tft.setCursor(4,10);                      // debug - print value below bat symbol
  tft.setTextSize(1);                       // debug
  tft.setTextColor(color[bValue]);          // debug
  tft.print(bValue);                        // debug
}
  



//******************* Check Radio Function ***********************************
// This function checks radio and process data, compares it to last data
int checkRadio(){
  uint8_t buf[RF69_MAX_DATA_LEN];              // Reciever Buffer
  uint8_t buflen = radio.DATALEN;          // Buffer Length
  int flag=0;
  int value[10];                                // Integer Value
  int count = numberof+1;                       // Count of data
  char valueChar[4];                            // Char Value

//  if (millis() > (comTimer + numberof*1000 + 1000)) {         // Check Tranmit timer
//    comTimer = millis();
//  }
  if (radio.receiveDone()){
    comTimer = millis()+300000;                   // com timer
    for(int i=0; i<buflen; i++){
      buf[i]=radio.DATA[i];
    }
    for(int j=0; j<3; j++){                       // data has 3 digits
      valueChar[j]=char(buf[j]);                  // make ascii char values
    }
    value[0] = atoi(valueChar)-100;               // convert char to int
//    comTimer = millis()-(value[0]*1000);          // start com timer
    Serial.print("Values Recieved: ");            // Debug
      for (int i = 0; i < buflen; i++) {
        Serial.print(char(buf[i]));
        if (i==2||i==5||i==8||i==11||i==14||i==17) Serial.print(" ");
      }
      Serial.println("");
    caTimer = (millis() + (numberof*1000 + 1000));
    for (int i=0; i<count; i++){                  // for message data count
      for(int j=0; j<3; j++){                     // data has 3 digits
        valueChar[j]=char(buf[j+(i*3)]);          // make ascii char values
      }
      value[i] = atoi(valueChar)-100;             // convert char to int
    }
    for (int i=0; i<=numberof; i++){              // for sensor values in message
      if (value[i]<222){                            // only overwrite unknown values
        sv[i] = value[i];                         // put sensor values in aray
      }
    }
       
    for (int i=0; i<count; i++){
      
      valueChar[1]=0;                             // clear array 
    } 
    flag=1;
  }
  return flag;
}
//******************* Report Function ***********************************
// This function sends command to report
void Report(){
  char numChar[4];                      // Char number
  int count = numberof+1;               // Count of data numbers
  int sn=0;                             // Int sensor number
  int num[50];                          // Int combined number array
  char msg[50];                         // Char combined number for message 

  num[0]=sensorNumber;                    // add sensor number to int array       
  for (int i=1; i<count; i++) {           // build int array 
    num[i] = 222   ;                      // add undknown sensor value
  }
  for (int i=0; i<count; i++){            // build char array
    itoa(num[i]+100, numChar, 10);        // add 100 to value and convert int to char
      for (int j=0; j<3; j++){            // for each 3 digit char
      msg[j+(i*3)] = numChar[j];          // put char into message
    }
  }
  msg[count*3]='\0';                      // end msg with null
  Serial.println("Report");               // debug
  radio.send(RECEIVER,msg, strlen(msg)); //target node Id, message as string or byte array, message length
 // vw_send((uint8_t *)msg, strlen(msg));   // send message

  // vw_rx_start();                          // Start receiver       
  radio.receiveDone(); //put radio in RX mode
  Serial.flush(); //make sure all serial data is clocked out before sleeping the MCU
}
//******************* Reset Function ***********************************
// This function sends command to reset
void Reset(){
  char numChar[4];                      // Char number
  int count = numberof+1;               // Count of data numbers
  int sn=0;                             // Int sensor number
  int num[50];                          // Int combined number array
  char msg[50];                         // Char combined number for message 
  
  num[0]=sensorNumber;                    // add sensor number to int array       
  for (int i=1; i<count; i++) {           // build int array 
    num[i] = 223   ;                      // add sensor Reset value
  }
  for (int i=0; i<count; i++){            // build char array
    itoa(num[i]+100, numChar, 10);        // add 100 to value and convert int to char
      for (int j=0; j<3; j++){            // for each 3 digit char
      msg[j+(i*3)] = numChar[j];          // put char into message
    }
  }
  msg[count*3]='\0';                      // end msg with null
  Serial.println("Reset");                // debug
  radio.send(RECEIVER,msg, strlen(msg)); //target node Id, message as string or byte array, message length
  // vw_send((uint8_t *)msg, strlen(msg));   // send message

  // vw_rx_start();                          // Start receiver 
  radio.receiveDone(); //put radio in RX mode
  Serial.flush(); //make sure all serial data is clocked out before sleeping the MCU
}
//******************* Vibrate Function ***********************************
// This function buzzes out active sensor number
void Vibrate(int count){
  int buzz[]={0,0,0,0};
  buzz[0]=count & 1;
  buzz[1]=count & 2;
  buzz[3]=count & 4;
  buzz[4]=count & 8;
  
  

  
}

//******************* Measure Battery Function ***********************************
// This function measures battery using Feather built-in voltage divider on A9
int batLevel(){
  int battery = 0; 
  float measuredvbat = analogRead(VBATPIN);
  measuredvbat *= 2;    // we divided by 2, so multiply back
  measuredvbat *= 3.3;  // Multiply by 3.3V, our reference voltage
  measuredvbat /= 1024; // convert to voltage
  if (measuredvbat >= 4) battery = 4;                         // Battery full
  if (measuredvbat < 4 && measuredvbat >= 3.6) battery = 3;   // Battery normal
  if (measuredvbat < 3.6 && measuredvbat >= 3.4) battery = 2; // getting low
  if (measuredvbat <3.4) battery = 1;                         // almost dead
  
  Serial.print("VBat: " ); Serial.println(measuredvbat);      // debug
  return battery;
}

  
